package com.younike.admin.aspect;

import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.younike.admin.manager.AsyncManager;
import com.younike.admin.manager.factory.AsyncFactory;
import com.younike.admin.utils.ShiroUtils;
import com.younike.common.annotation.OpLog;
import com.younike.common.constant.BusinessStatus;
import com.younike.common.utils.json.JSON;
import com.younike.common.utils.servlet.ServletUtils;
import com.younike.common.utils.string.StringUtils;
import com.younike.service.pojo.SysOperLog;
import com.younike.service.pojo.SysUser;
import org.aspectj.lang.JoinPoint;
import org.aspectj.lang.Signature;
import org.aspectj.lang.annotation.AfterReturning;
import org.aspectj.lang.annotation.AfterThrowing;
import org.aspectj.lang.annotation.Aspect;
import org.aspectj.lang.annotation.Pointcut;
import org.aspectj.lang.reflect.MethodSignature;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.stereotype.Component;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import java.lang.reflect.Method;
import java.util.Map;

/**
 * 操作日志记录处理
 *
 * @author Andrew
 */
@Aspect
@Component
public class OptLogAspect {
    private static final Logger log = LoggerFactory.getLogger(OptLogAspect.class);

    /**
     * 配置切入点
     */
    @Pointcut("@annotation(com.younike.common.annotation.OpLog)")
    public void logPointCut() {
    }

    /**
     * 处理完请求后执行
     *
     * @param joinPoint 切点
     */
    @AfterReturning(pointcut = "logPointCut()", returning = "jsonResult")
    public void doAfterReturning(JoinPoint joinPoint, Object jsonResult) {
        handleLog(joinPoint, null, jsonResult);
    }

    /**
     * 拦截异常操作
     *
     * @param joinPoint 切点
     * @param e         异常
     */
    @AfterThrowing(value = "logPointCut()", throwing = "e")
    public void doAfterThrowing(JoinPoint joinPoint, Exception e) {
        handleLog(joinPoint, e, null);
    }

    /**
     * 处理日志
     *
     * @param joinPoint  切点
     * @param e          异常
     * @param jsonResult 结果集
     */
    protected void handleLog(final JoinPoint joinPoint, final Exception e, Object jsonResult) {
        try {
            // 获得注解
            OpLog controllerLog = getAnnotationLog(joinPoint);
            if (controllerLog == null) {
                return;
            }

            // 获取当前的用户
            SysUser currentUser = ShiroUtils.getSysUser();

            // *========数据库日志=========*//
            SysOperLog operLog = new SysOperLog();
            operLog.setStatus(BusinessStatus.SUCCESS.ordinal());
            // 请求的地址
            String ip = ShiroUtils.getIp();
            operLog.setOperIp(ip);
            // 返回参数
            operLog.setJsonResult(JSON.marshal(jsonResult));

            operLog.setOperUrl(ServletUtils.getRequest().getRequestURI());
            if (currentUser != null) {
                operLog.setOperName(currentUser.getLoginName());
            }

            JSONObject jsonObject = JSONArray.parseObject(operLog.getJsonResult());
            if (e != null || (jsonObject != null && jsonObject.getString("code") != null &&
                    "500".equals(jsonObject.getString("code")))) {
                operLog.setStatus(BusinessStatus.FAIL.ordinal());
                String msg = "";
                if (e != null) {
                    msg = StringUtils.substring(e.getMessage(), 0, 2000);
                }
                if (jsonObject != null && jsonObject.getString("msg") != null) {
                    msg = jsonObject.getString("msg");
                }
                operLog.setErrorMsg(msg);
            }
            // 设置方法名称
            String className = joinPoint.getTarget().getClass().getName();
            String methodName = joinPoint.getSignature().getName();
            Object[] args = joinPoint.getArgs();
            operLog.setMethod(className + "." + methodName + "()");
            // 设置请求方式
            operLog.setRequestMethod(ServletUtils.getRequest().getMethod());
            // 处理设置注解上的参数
            getControllerMethodDescription(controllerLog, operLog, args);
//            // 保存数据库
            AsyncManager.me().execute(AsyncFactory.recordOper(operLog));
        } catch (Exception exp) {
            // 记录本地异常日志
            log.error("==前置通知异常==");
            log.error("异常信息:{}", exp.getMessage());
            exp.printStackTrace();
        }
    }

    /**
     * 获取注解中对方法的描述信息 用于Controller层注解
     *
     * @param log     日志
     * @param operLog 操作日志
     * @throws Exception
     */
    public void getControllerMethodDescription(OpLog log, SysOperLog operLog, Object[] args) throws Exception {
        // 设置action动作
        operLog.setBusinessType(log.type().ordinal());
        // 设置标题
        operLog.setTitle(log.title());
        // 设置操作人类别
        operLog.setOperatorType(log.operatorType().ordinal());
        // 是否需要保存request，参数和值
        if (log.isSaveRequestData()) {
            // 获取参数的信息，传入到数据库中。
            setRequestValue(operLog, args);
        }
    }

    /**
     * 获取请求的参数，放到log中
     *
     * @param operLog 操作日志
     * @throws Exception 异常
     */
    private void setRequestValue(SysOperLog operLog, Object[] args) throws Exception {
        HttpServletRequest request = ServletUtils.getRequest();
        String params = "";
        if ("POST".equalsIgnoreCase(request.getMethod())) {
            // 获取@RequestBody信息
            if (args != null && args.length > 0) {
                for (int i = 0; i < args.length; i++) {
                    if (!(args[i] instanceof HttpServletRequest)
                            && !(args[i] instanceof HttpServletResponse)) {
                        params = JSON.marshal(args[i]);
                    }
                }
            }
        } else {
            Map<String, String[]> map = request.getParameterMap();
            params = JSON.marshal(map);
        }
        operLog.setOperParam(StringUtils.substring(params, 0, 2000));
    }

    /**
     * 是否存在注解，如果存在就获取
     */
    private OpLog getAnnotationLog(JoinPoint joinPoint) {
        Signature signature = joinPoint.getSignature();
        MethodSignature methodSignature = (MethodSignature) signature;
        Method method = methodSignature.getMethod();

        if (method != null) {
            return method.getAnnotation(OpLog.class);
        }
        return null;
    }
}
